*! version 1.0.4 PR 10mar2006
* 1.0.4 10mar2006: fix bug in parameterwise option, not included as part of jackknife calcs.
*		   add stepwise as synomym for sw.
program define cval, rclass sortpreserve
version 8
gettoken cmd 0 : 0
local fp 0
if "`cmd'"=="mfp" | "`cmd'"=="fracpoly" | "`cmd'"=="stepwise" | "`cmd'"=="sw" | "`cmd'"=="mvrs" {
	if"`cmd'"=="mfp" | "`cmd'"=="fracpoly" | "`cmd'"=="mvrs" {
		local fp 1
	}
	* mbcmd denotes model-building command
	local mbcmd `cmd'
	gettoken cmd 0 : 0
}

if "`cmd'"=="stpm" {
	local dist 7
	local glm 0
	local qreg 0
	local xtgee 0
	local normal 0
	local eqxb [xb]
}
else {
	* currently supported commands
	local cmds clogit cnreg cox ereg fit glm logistic logit ologit oprobit poisson probit /*
	 */ qreg regress rreg weibull xtgee streg stcox stpm nbreg
	cmdchk `cmd' `cmds'
	if `s(bad)' {
		di as error "invalid or unrecognised command, `cmd'"
		exit 198
	}
	/*
		dist=0 (normal), 1 (binomial), 2 (poisson), 3 (cox), 4 (glm),
		5 (xtgee), 6(ereg/weibull), 7 (streg/stpm).
	*/
	local dist `s(dist)'
	local glm `s(isglm)'
	local qreg `s(isqreg)'
	local xtgee `s(isxtgee)'
	local normal `s(isnorm)'
	local eqxb
}

if "`cmd'"!="cox" & "`cmd'"!="stcox" & "`cmd'"!="streg" { // streg should allow noconstant, but does not work with offset()
	local vl varlist(min=1 numeric)
	local kons noconstant	// allows noconstant in calculation of out-of-sample deviance
}
else local vl [varlist(default=none)]
syntax `vl' [if] [in] [aweight fweight pweight iweight], ///
 [, BLock(string) SHrink(real 1) Dead(string) FOld(int 0) LEAveout(int 1) ///
    REPs(int 1) SEed(str) XB DIchot(string) MBopts (string) noINSample PARAMeterwise *]

if `shrink'>1 {
	di as err "shrink() must not exceed 1"
	exit 198
}
if `shrink'<1 & "`dichot'"!="" {
	di as err "shrink()<1 not allowed with dichot()"
	exit 198
}
if "`mbcmd'"!="" & "`parameterwise'"!="" {
	di as err "parameterwise option not allowed with model-building commands"
	exit 198
}

if "`block'"!="" {
	unab block: `block'
}

if `fp' {
	local mbopts `mbopts' all	// to allow out-of-sample prediction for FP models
}
local isbinary="`cmd'"=="logit" | "`cmd'"=="logistic" | "`cmd'"=="probit"

if "`dichot'"!="" {
	if "`cmd'"=="mlogit" | "`cmd'"=="ologit"{
		di as err "dichot() not yet available with `cmd'"
		exit 198
	}
	* dichot is used to dichotomize the predicted probabilities for calculating the 0-1 loss function.
	if lower("`dichot'")=="mean" {
		local dichot mean
	}
	else confirm number `dichot'
}
if `fold'==1 | `fold'<0 {
	di as err "invalid fold(), must be at least 2"
	exit 198
}
if `fold'>0 & `leaveout'!=1 {
	di as text "[fold() specified, ignoring leaveout(`leaveout')]"
	local leaveout 1
}
if `reps'>1 & `leaveout'==1 & `fold'==0 {
	di as text "[reps(`reps') ignored since doing leave-one-out cross-validation]"
	local reps 1
}
if "`seed'"!="" {
	set seed `seed'
}
marksample touse
markout `touse' `varlist' `dead' `block'
local wt 1
if "`weight'"!="" {
	if "`weight'"=="fweight" {
		preserve
		expand `exp'
		local fwvar: word 2 of `exp'
		drop if `fwvar'==0
	}
	else {
		if `dist'==7 {
			di as err "weights not allowed"
			exit 198
		}
		tempvar wt
		qui gen double `wt' `exp'
		if "`weight'"=="aweight" | "`weight'"=="pweight" {
			* standardize these weights to have mean 1
			qui sum `wt' if `touse', meanonly
			qui replace `wt'=`wt'/r(mean)
		}
		local w [`weight'=`wt']
	}
}
local jack=(`normal' & "`mbcmd'"=="" & `leaveout'==1 & `fold'==0 ///
 & "`w'"=="" & "`block'"=="" & "`parameterwise'"=="")	// normal-errors jackknife calcs available

qui if "`block'"!="" { // "blocks" are clusters for cross-validation
	tempvar Block first
	egen long `Block'=group(`block') if `touse'==1
	compress `Block'
	sum `Block' if `touse', meanonly
	local Nobs=r(max)
	bysort `Block': gen byte `first'=(_n==1)	// points to first obs in each block
	local blocks blocks
}
	
qui count if `touse'
local nobs=r(N)
if "`block'"=="" {
	local Nobs `nobs'
	local blocks obs
}
if `Nobs'<2 {
	if `Nobs'==0 {
		error 2000
	}
	error 2001
}
if `fold'>0 {
	local leaveout=int(`Nobs'/`fold'+0.5)
	di as text "[`fold'-fold cross-validation, leaving out batches of about `leaveout' `blocks'.]"
}
if `leaveout'<1 | `leaveout'>`nobs' {
	di in red "invalid leaveout()"
	exit 198
}
if `dist'==7 {
	local rhs `varlist'
}
else gettoken lhs rhs: varlist
if "`dead'"!="" {
	local dead dead(`dead')
}

* check rhs for collinearity
tempname b f V
quietly matrix accum `V'=`rhs' `w' if `touse'
matrix `V'=syminv(`V')
tokenize `rhs'
local i 1
while "``i''"!="" {
	if `V'[`i',`i']==0 {
		di as text "note: ``i'' dropped due to collinearity"
		local `i' " "
	}
	local i=`i'+1
}
matrix drop `V'
local rhs "`*'"
tempvar tmp
qui if "`xb'"!="" {
	tempvar XB
	gen double `XB'=0 if `touse'
	lab var `XB' "Cross-validated index for `lhs'"
	if "`dichot'"!="" {
		tempvar XBclass
		gen int `XBclass'=0 if `touse'
		lab var `XBclass' "Cross-validated class vote for `lhs'"
	}
}

if "`dichot'"=="mean" {
	qui sum `lhs' if `touse', meanonly
	local dichot=r(mean)
}
local calcshrink=("`dichot'"=="" & "`rhs'"!="")
tempname err cv sumcv ssqcv sumshr ssqshr

* Trial run of cmd to check that options are correctly specified
capture `cmd' `lhs' `rhs' if `touse' `w', `dead' `options'
local rc=_rc
if `rc' {
	di as err "error `rc' encountered, please check the syntax and options you specified"
	exit `rc'
}
* Null deviance
if "`cmd'"=="stcox" local esti estimate
qui `cmd' `lhs' if `touse' `w', `dead' `options' `esti'
local dev0=-2*e(ll)

quietly if "`insample'"!="noinsample" {
	`mbcmd' `cmd' `lhs' `rhs' if `touse' `w', `dead' `options' `mbopts'
	* Display model
	if "`mbcmd'"!="" noi `mbcmd'
	else noi `cmd'
	if "`xb'"!="" {
		cap drop _orig
		predict _orig if `touse', xb
		// !! remove centering of index - ?? ok for cox model
/*
		summarize _orig if `touse', meanonly
		replace _orig=_orig-r(mean)
*/
		lab var _orig "in-sample index"
	}
	if "`dichot'"!="" {
		* Misclassification rate in original sample
		tempvar predclass
		if `isbinary' {
			predict `tmp' if `touse'==1, p	// OOS probability of event
		}
		else predict `tmp' if `touse'==1, xb	// for discriminant function case
		gen byte `predclass'=(`tmp'>`dichot') if !missing(`tmp')
		drop `tmp'
		count if `predclass'!=`lhs' & `touse'==1
		scalar `err'=r(N)/`nobs'	// overall in-sample misclassification rate
	}
	else {
		* Deviance in original sample
		scalar `err'=-2*e(ll)
	}
}
else scalar `err'=.
quietly {
	if "`parameterwise'"!="" {
		* Cross-validated index for each covariate
		local xvars
		local i 0
		tokenize "`rhs'"
		while "`1'"!="" {
			local ++i
			local v`i' `1'
			tempvar x`i' xb`i'
			gen double `x`i''=`1' if `touse'
			sum `x`i'', meanonly
			replace `x`i''=`x`i''-r(mean) // standardize each covariate to mean zero
			local xvars `xvars' `x`i''
			gen `xb`i''=.
			mac shift
		}
		local nv `i'
		local rhs `xvars'
	}
	tempvar u excl order index
	gen long `order'=.
	gen byte `excl'=.
	gen double `index'=.	// cross-validated index
	scalar `sumcv'=0
	scalar `ssqcv'=0
	if `calcshrink' {
		scalar `sumshr'=0
		scalar `ssqshr'=0
	}
	forvalues r=1/`reps' {
		if `jack' {
			* jackknifed unweighted normal errors regression short-cuts the calcs
			* (see e.g. Hastie & Tibshirani 1990 page 47 eq. 3.18)
			tempvar hat
			predict double `hat' if `touse'==1, hat
			predict double `tmp' if `touse'==1, res
			replace `index'=`lhs'-`tmp'/(1-`hat')	// jackknifed index
			if "`dichot'"!="" {
				replace `predclass'=(`index'>`dichot') if `touse'==1
			}
			drop `tmp' `hat'
		}
		else {
			scalar `cv'=0
			gen `u'=uniform()
			gsort -`touse' `first' `u'
			drop `u'
			replace `order'=_n if `touse'	// keep order in case `cmd' reorders data later
			local i 1
			local j1 0
			local j2 0
			local pc10=int(.1*`Nobs'+.5)
			while `i'<=`Nobs' {
				local i1=`i'+`leaveout'-1
				if `i1'>`Nobs' {
					local i1 `Nobs'
				}
				else if (`Nobs'-`i1')<`leaveout'/2 {
					local i1 `Nobs'
				}
				* Predict index or probability in excluded part
				if "`block'"=="" {
					replace `excl'=(`order'>=`i' & `order'<=`i1')
				}
				else replace `excl'=(`Block'>=`i' & `Block'<=`i1')
				`mbcmd' `cmd' `lhs' `rhs' if `excl'==0 & `touse'==1 `w', ///
				 `dead' `options' `mbopts'
				predict `tmp' if `touse'==1, xb
				replace `index'=`tmp' if `excl'==1 & `touse'==1

				// !! Unclear if standardization should be applied or not.
/*
				* Standardize the index with mean from the estimation sample
				summarize `tmp' if `excl'==0, meanonly
				replace `index'=`tmp'-r(mean) if `excl'==1 & `touse'==1
*/
				if "`dichot'"!="" {
					if `isbinary' {
						drop `tmp'
						predict `tmp' if `excl'==1 & `touse'==1, p	// OOS probability of event
					}
					replace `predclass'=(`tmp'>`dichot') if `excl'==1 & `touse'==1 & !missing(`tmp')
				}
				if "`parameterwise'"!="" {
					// !! not currently being averaged over reps - final rep only
					// !! won't work with mfp etc where new covariates are created
					forvalues l=1/`nv' {
						replace `xb`l''=`eqxb'_b[`x`l'']*`x`l'' if `excl'==1 & `touse'==1
					}
				}
				local i=`i1'+1
				local j1=`j1'+`leaveout'
				if `j1'>=`pc10' {
					local j1 0
					local j2=`j2'+10
					noi di as text %4.0f `j2' "%" _cont
				}
				drop `tmp'
			}
		}
		if `shrink'<1 {
			* Centre index and apply shrinkage factor
			sum `index' if `touse'==1, meanonly
			replace `index'=r(mean)+`shrink'*(`index'-r(mean))
		}
		if `calcshrink' {
			* compute shrinkage for current index and update for mean and SE
			`cmd' `lhs' `index' if `touse'==1 `w', `dead' `options'
			local shr=`eqxb'_b[`index']
			scalar `sumshr'=`sumshr'+`shr'
			scalar `ssqshr'=`ssqshr'+`shr'^2
		}
		if "`xb'"!="" {
			replace `XB'=`XB'+`index' if `touse'==1
			if "`dichot'"!="" {
				replace `XBclass'=`XBclass'+`predclass' if `touse'==1
			}
		}
		* Calculate loss function (deviance or misclassification rate) out of sample (OOS)
		if "`dichot'"!="" {
			count if `predclass'!=`lhs' & `touse'==1
			scalar `cv'=r(N)/`nobs'	// misclassification rate
		}
		else {
			if `isbinary' {
				* Direct calculation of deviance for logistic and probit models
				if "`cmd'"=="probit" {
					replace `index'=norm(`index') if `touse'==1
				}
				else {
					replace `index'=1/(1+exp(-`index')) if `touse'==1
				}
				replace `index'=`wt'*cond(`lhs'==1, ln(`index'), ln(1-`index')) if `touse'==1 // LL
				sum `index' if `touse'==1 , meanonly
				scalar `cv'=-2*r(sum)	// deviance
			}
			else {
				if `normal' {
					* calc deviance via residuals and (weighted) null regression
					* !! removing noconstant, since have standardized the index
					gen double `tmp'=`lhs'-`index' if `touse'==1
					*`cmd' `tmp' `w', noconstant `options'
					`cmd' `tmp' `w', `options'
					drop `tmp'
				}
				else {
					if "`cmd'"=="stcox" local esti estimate
					// !! not clear if need or don't need "noconstant" (`kons') in the following statement.
					*`cmd' `lhs' if `touse'==1 `w', `dead' `options' offset(`index') `kons' `esti'
					`cmd' `lhs' if `touse'==1 `w', `dead' `options' offset(`index') `esti'
				}
				scalar `cv'=-2*e(ll)
			}
		}
		scalar `sumcv'=`sumcv'+`cv'
		scalar `ssqcv'=`ssqcv'+`cv'^2
		noi di ".." `r'
	}
}
if "`xb'"!="" {
	cap drop _cval
	gen double _cval=`XB'/`reps'	// mean out-of-sample (cross-validated) index over all replicates
	lab var _cval "Cross-validated index"
	if "`dichot'"!="" {
		cap drop _predclass
		rename `XBclass' _predclass	// at final replicate
	}
}
if "`parameterwise'"!="" {
	forvalues l=1/`nv' {
		cap drop XB`v`l''
		rename `xb`l'' XB`v`l''
		lab var XB`v`l'' "Cross-validated index for `v`l''"
	}
}
return scalar n=`nobs'
return scalar N=`Nobs'
return scalar dev0=`dev0'	// deviance for null model
return scalar err=`err'	// misclassification rate
if `reps'<2 {
	return scalar cv=`cv'
}
else {
	return scalar sd_cv=sqrt((`ssqcv'-`sumcv'^2/`reps')/(`reps'-1))
	return scalar se_cv=return(sd_cv)/sqrt(`reps')
	return scalar cv=`sumcv'/`reps'
}
return scalar opt=return(cv)-return(err)
if `calcshrink' {
	if `reps'>1 {
		return scalar sd_shrink=sqrt((`ssqshr'-`sumshr'^2/`reps')/(`reps'-1))
		return scalar se_shrink=return(sd_shrink)/sqrt(`reps')
	}
	return scalar shrink=`sumshr'/`reps'
}
local col 28
di _n as text "Number of observations" _col(`col') %9.0g as res return(n)
if "`block'"!="" {
	di as text "Number of blocks" _col(`col') %9.0g as res return(N)
}
di as text "In-sample error rate" _col(`col') %9.0g as res return(err)
di as text "Cross-validated error rate" _col(`col') %9.0g as res return(cv)
di as text "Optimism" _col(`col') %9.0g as res return(opt)
if `calcshrink' {
	di as text "Shrinkage" _col(`col') %9.0g as res return(shrink)
}
end

*! version 1.0.4 PR 30sep2005
* Based on private version 6.1.4 of frac_chk, PR 25aug2004
program define cmdchk, sclass
	version 7
	local cmd `1'
	mac shift
	local cmds `*'
	sret clear
	if substr("`cmd'",1,3)=="reg" {
		local cmd regress
	}
	if "`cmds'"=="" {
		tokenize clogit cnreg cox ereg fit glm logistic logit poisson probit /*
		*/ qreg regress rreg weibull xtgee streg stcox stpm stpmrs /*
		*/ ologit oprobit mlogit nbreg
	}
	else tokenize `cmds'
	sret local bad 0
	local done 0
	while "`1'"!="" & !`done' {
		if "`1'"=="`cmd'" {
			local done 1
		}
		mac shift
	}
	if !`done' {
		sret local bad 1
		*exit
	}
	/*
		dist=0 (normal), 1 (binomial), 2 (poisson), 3 (cox), 4 (glm),
		5 (xtgee), 6 (ereg/weibull), 7 (stcox, streg, stpm, stpmrs).
	*/
	if "`cmd'"=="logit" | "`cmd'"=="probit" /*
 	*/ |"`cmd'"=="clogit"| "`cmd'"=="logistic" /*
 	*/ |"`cmd'"=="mlogit"| "`cmd'"=="ologit" | "`cmd'"=="oprobit" {
						sret local dist 1
	}
	else if "`cmd'"=="poisson" {
						sret local dist 2
	}
	else if "`cmd'"=="cox" {
						sret local dist 3
	}
	else if "`cmd'"=="glm" {
						sret local dist 4
	}
	else if "`cmd'"=="xtgee" {
						sret local dist 5
	}
	else if "`cmd'"=="cnreg" | "`cmd'"=="ereg" | "`cmd'"=="weibull" | "`cmd'"=="nbreg" {
						sret local dist 6
	}
	else if "`cmd'"=="stcox" | "`cmd'"=="streg" | "`cmd'"=="stpm" | "`cmd'"=="stpmrs" {
						sret local dist 7
	}
	else if substr("`cmd'",1,2)=="st" {
						sret local dist 7
	}
	else					sret local dist 0

	sret local isglm  = (`s(dist)'==4)
	sret local isqreg = ("`cmd'"=="qreg")
	sret local isxtgee= (`s(dist)'==5)
	sret local isnorm = ("`cmd'"=="regress"|"`cmd'"=="fit"|"`cmd'"=="rreg") 
end
