*! version 1.1.1 PR 29may2007
program define gdelta, rclass
version 8
syntax newvarname =/exp [if] [in], [ Epsilon(real 0.01) noStandardize Delta(real 0.2) meansd(string) Rescale ]
if `epsilon'<=0 {
	di as err "invalid epsilon(), must be positive"
	exit 198
}
if `delta'<0 {
	di as txt "[note: delta ignored, no origin shift applied]"
}
if "`meansd'"!="" {
	gettoken m s: meansd
	confirm number `m'
	confirm number `s'
}
tempvar x
quietly {
	tempname mean sd min max
	gen double `x' = `exp' `if' `in'
	sum `x'
	scalar `min'=r(min)
	scalar `max'=r(max)
	if "`meansd'"!="" {	// take user-supplied mean and sd, both required
		scalar `mean'=`m'
		scalar `sd'=`s'
	}
	else {
		if "`standardize'"!="nostandardize" {
			scalar `mean'=r(mean)
			scalar `sd'=r(sd)
		}
		else {
			scalar `mean'=0
			scalar `sd'=1
		}
	}
	replace `x'=norm((`x'-`mean')/`sd')
	tempname theta
	scalar `theta'=-ln(`epsilon'/(1+`epsilon'))
	gen double `varlist'=(ln((`x'+`epsilon')/(1+`epsilon'-`x'))+`theta')/(2*`theta')
	if `delta'>=0 {
		tempname gmin gmax
		sum `varlist'
		scalar `gmin'=r(min)
		scalar `gmax'=r(max)
		replace `varlist'=`delta'+(1-`delta')*(`varlist'-`gmin')/(`gmax'-`gmin')
		local o ", delta = `delta'"
	}
	if "`rescale'"!="" {
		* rescale gdelta() such that mean and SD of gdelta()
		* are comparable to those of x in the central region
		Rescale `varlist' `delta' `mean' `sd'
	}
	lab var `varlist' "gdelta(`exp'), epsilon=`epsilon'`o'"
}
* determine low and high limits beyond which extrapolation is unwise
return scalar low =max(`min', `mean'-2.783*`sd')
return scalar high=min(`max', `mean'+2.783*`sd')
* determine these values on the transformed scale
tempname slow shigh glow ghigh
scalar `slow' =norm((return(low)-`mean')/`sd')
scalar `shigh'=norm((return(high)-`mean')/`sd')
scalar `glow' =(ln((`slow'+`epsilon')/(1+`epsilon'-`slow'))+`theta')/(2*`theta')
scalar `ghigh'=(ln((`shigh'+`epsilon')/(1+`epsilon'-`shigh'))+`theta')/(2*`theta')
if `delta'>0 {
	scalar `glow'=`delta'+(1-`delta')*(`glow'-`gmin')/(`gmax'-`gmin')
	scalar `ghigh'=`delta'+(1-`delta')*(`ghigh'-`gmin')/(`gmax'-`gmin')
}
if "`rescale'"!="" {
	Rescale `glow'  `delta' `mean' `sd'
	Rescale `ghigh' `delta' `mean' `sd'
}
return scalar glow =`glow'
return scalar ghigh=`ghigh'
end

program define Rescale
args arg delta mean sd
tempname bz
scalar `bz'=0.143424	// determined empirically by linear regrn of gdelta() on z in +/-2.683 with n=1,000,000
quietly {
	cap confirm var `arg'
	if _rc==0 local replace replace
	else local replace scalar
	`replace' `arg' = `mean' + `sd'*(`arg'-(1+`delta')/2)/`bz'
}
end
