*! version 1.0.0 PR 07Aug2002.
program define growth, eclass
* Fits logistic or normal integral function for general response models
version 6
if replay() {
	if "`e(cmd)'"=="" | "`e(fp_cmd2)'"!="growth" {
		error 301
	}
	di in blue "->`e(cmd)'"
	_rslt
	exit
}
gettoken cmd 0 : 0
frac_chk `cmd' 
if `s(bad)' {
	di in red "invalid or unrecognised command, `cmd'"
	exit 198
}
/*
	dist=0 (normal), 1 (binomial), 2 (poisson), 3 (cox), 4 (glm),
	5 (xtgee), 6(ereg/weibull).
*/
local dist `s(dist)'
local glm `s(isglm)'
local qreg `s(isqreg)'
local xtgee `s(isxtgee)'
local normal `s(isnorm)'
/*
	dist=0 (normal), 1 (binomial), 2 (poisson), 3 (cox), 4 (glm),
	5 (xtgee), 6(ereg/weibull).
*/
syntax varlist(min=2) [if] [in] [aw fw pw iw] [, /* 
 */ noCONStant COVars(varlist) DEAD(varname) ITer(int 100) LOGistic /*
 */ LTOLerance(real .0001) MINit(string) SINit(string) TRace * ]
if "`logisti'"=="" {	/* default is Normal integral transform */
	local nprob nprob
}
frac_cox "`dead'" `dist'
if "`trace'"!="" { local trace noi }
local nbase: word count `covars'
if "`constan'"=="noconstant" {
	if "`cmd'"=="fit" | "`cmd'"=="cox" {
		di in red "noconstant invalid with `cmd'"
		exit 198
	}
	loc options "`options' nocons"
}
tempname small
scalar `small'=1e-6
tokenize `varlist'
local y `1'
mac shift
local rhs `*'
tempvar touse
quietly {
	marksample touse
	markout `touse' `y' `rhs' `covars' `dead'
/*
	Deal with weights.
*/
	frac_wgt `"`exp'"' `touse' `"`weight'"'
	local mnlnwt = r(mnlnwt) /* mean log normalized weights */
	local wgt `r(wgt)'
	if "`dead'"!="" { local dead "dead(`dead')" }

	tokenize `rhs'
	local nx 0
	while "`1'"!="" {
		local nx=`nx'+1
		local v`nx' `1'
		frac_mun `1' purge	/* Remove all old I* variables */
		mac shift
	}
/*
	Initialise params (two options)
*/
	if "`minit'"!="" {
		frac_ext "`rhs'" "`minit'" init
		local v 0
		while `v'<`nx' {
			local v=`v'+1
			local minit`v' ${S_`v'}
		}
	}
	if "`sinit'"!="" {
		frac_ext "`rhs'" "`sinit'" init
		local v 0
		while `v'<`nx' {
			local v=`v'+1
			local sinit`v' ${S_`v'}
		}
	}
/*
	Initialise
*/
	local v 0
	while `v'<`nx' {
		local v=`v'+1
		tempname m`v' s`v' xb`v' 	/* params for v'th variable */
		sum `v`v'' if `touse'
		scalar `xb`v''=r(mean)
		if "`nprob'"=="" {
			scalar `s`v''=-1/r(sd)
			scalar `m`v''=0
		}
		else {
			scalar `s`v''=1/r(sd)
			scalar `m`v''=-0
		}
		if "`minit`v''"!="" {
			confirm num `minit`v''
			scalar `m`v''=`minit`v''
		}
		if "`sinit`v''"!="" {
			confirm num `sinit`v''
			scalar `s`v''=`sinit`v''
		}
	}
	count if `touse'
	local nobs=r(N)
/*
	Determine residual and model df.
*/
	regress `y' `covars' `wgt' if `touse'
	local rdf=e(df_r)+("`constan'"=="noconstant")
/*
	Calc deviance=-2(log likelihood) for regression on covars only,
	allowing for possible weights.

	Note that for logit/clogit/logistic with nocons, must regress on zero,
	otherwise get r(102) error.
*/
	if `dist'==1 & "`constan'"=="noconstant" {
		tempvar z0
		gen `z0'=0
	}
	`cmd' `y' `z0' `covars' `wgt' if `touse', `dead' `options'
	if `xtgee' & "`covars'"=="" { global S_E_chi2 0 }
	local glmwarn 0
	if `glm' {
	 local scale 1
	 if abs($S_E_dc/$S_E_del-1)>`small' & abs($S_E_dd/$S_E_del-1)>`small' {
		local scale $S_E_del
	 }
	 else local glmwarn 1
	}
	frac_dv `normal' "`wgt'" `nobs' `mnlnwt' `dist' /*
	 */ `glm' `xtgee' `qreg' "`scale'"
	local dev0=r(deviance)
/*
	Calc first working variables w`v'
*/
	local wvars
	local v 0
	while `v'<`nx' {
		local v=`v'+1
		tempvar w`v' z`v' z2`v'	/* F'(), x*F'() */
		if "`nprob'"=="" {
			gen double `w`v''= 1/(1+exp(-(`s`v''*(`v`v''-`xb`v'')+`m`v''))) if `touse'
		}
		else 	gen double `w`v''= norm(`s`v''*(`v`v''-`xb`v'')+`m`v'') if `touse'
		gen double `z`v''=.
		gen double `z2`v''=.
		local wvars `wvars' `w`v''
	}
/*
	Initial fit of whole model
*/
	`cmd' `y' `wvars' `covars' `wgt', `dead' `options'
	frac_dv `normal' "`wgt'" `nobs' `mnlnwt' `dist' /*
	 */ `glm' `xtgee' `qreg' "`scale'"
	local devnow=r(deviance)
	noi di in gr _n "Iteration 0:  Deviance = " in ye %9.0g `devnow'
	local v 0
	while `v'<`nx' {
		local v=`v'+1
		tempname b`v'
		scalar `b`v''=_b[`w`v'']	/* beta1 */
	}
/*
	Iterate
*/
	tempname mold sold mnew snew bz bz2 delta delta2 rev rev2
	tempvar wold
	gen double `wold'=.
	local devinit `devnow'
	local devprev `devnow'
	local devcon `ltolera' /* deviance convergence criterion */
	local converg 1
	local totalit 0
	local done 0
	local j 1
	while `j'<=`iter' & !`done' {
	    local v 0
	    while `v'<`nx' & !`done' {
		    local v=`v'+1
		    local devpr2 `devnow'
		    local done2 0
		    local j2 1
		    while `j2'<=`iter' & !`done2' {
			replace `wold'=`w`v''
			scalar `mold'=`m`v''
			scalar `sold'=`s`v''
			if "`nprob'"=="" {
				replace `z`v''= `wold'*(1-`wold') if `touse'
			}
			else replace `z`v''= normden(`sold'*(`v`v''-`xb`v'')+`mold') if `touse'
			replace `z2`v''=`z`v''*(`v`v''-`xb`v'')
/*
	 Regression for updating params
*/
			`cmd' `y' `wvars' `z`v'' `z2`v'' `covars' `wgt', `dead' `options'
			cap scalar `bz'=_b[`z`v'']
			if _rc {
				scalar `bz'=`b`v''
			}
			cap scalar `bz2'=_b[`z2`v'']
			if _rc {
				scalar `bz2'=`b`v''
			}
			scalar `delta'=`bz'/`b`v''
			scalar `delta2'=`bz2'/`b`v''
			local dun 0
			scalar `rev'=1	/* step-sign for m */
			scalar `rev2'=1	/* step-sign for s */
			local jfail 0
			local j3 1
			while `j3'<=`iter' & !`dun' {
				* Trial update
				scalar `mnew'=`mold'+`rev'*`delta'
				scalar `snew'=`sold'+`rev2'*`delta2'
				if "`nprob'"=="" {
					replace `w`v''= 1/(1+exp(-(`snew'*(`v`v''-`xb`v'')+`mnew'))) if `touse'
				}
				else 	replace `w`v''=norm(`snew'*(`v`v''-`xb`v'')+`mnew') if `touse'
				cap `cmd' `y' `wvars' `covars' `wgt', `dead' `options'
				local rc=_rc
				cap scalar `bz'=_b[`w`v'']
				local rc1=_rc
				if `rc'==0 & `rc1'==0 {
					frac_dv `normal' "`wgt'" `nobs' `mnlnwt' /*
					 */ `dist' `glm' `xtgee' `qreg' "`scale'"
					local devdiff=(r(deviance)-`devpr2')
					if "`trace'"!="" {
					    noi di in gr "`v`v''" %4.0f `j2' /*
					     */ _col(16) "mtry=" in ye %9.0g `mnew' /*
					     */ _col(28) in gr " stry=" in ye %9.0g `snew' /*
					     */ _col(44) %12.6f r(deviance) /*
					     */ _col(60) %9.6f `devdiff'
					}
					if `devdiff'<0 { /* accept new parameters */
						local devnow=r(deviance)
						scalar `b`v''=_b[`w`v'']
						scalar `m`v''=`mnew'
						scalar `s`v''=`snew'
						if `devdiff'>-`devcon' { local done2 1 }
						local dun 1
					}
					else {
						* halve step lengths and seek best direction
						local dd 1e30
						local j5 1
						while `dd'>`ltolera' & !`dun' { /* may accept at worst a small deterioration in deviance */
							local j5=`j5'+1
							if `j5'>15 {
								noi di in red "no convergence, cannot improve fit"
								exit 498
							}
							scalar `delta'=`delta'/2
							scalar `delta2'=`delta2'/2
							local improvd 0
							local j4 1
							while `j4'<=4 {
								* eval step+/- for m and s
								if `j4'<=2 { scalar `rev'=1 }
								else scalar `rev'=-1
								if `j4'==1 | `j4'==3 { scalar `rev2'=1 }
								else scalar `rev2'=-1
								scalar `mnew'=`mold'+`rev'*`delta'
								scalar `snew'=`sold'+`rev2'*`delta2'
								if "`nprob'"=="" {
									replace `w`v''= 1/(1+exp(-(`snew'*(`v`v''-`xb`v'')+`mnew'))) if `touse'
								}
								else 	replace `w`v''=norm(`snew'*(`v`v''-`xb`v'')+`mnew') if `touse'
								cap `cmd' `y' `wvars' `covars' `wgt', `dead' `options'
								frac_dv `normal' "`wgt'" `nobs' `mnlnwt' /*
								 */ `dist' `glm' `xtgee' `qreg' "`scale'"
*noi di in red "delta=" `delta' " delta2=" `delta2'
*noi di in red "v=`v' j=`j' j2=`j2' j3=`j3' j4=`j4' deviance=" %15.0g r(deviance) " dd=" r(deviance)-`devpr2'
								if (r(deviance)-`devpr2')<`dd' {
									local dd=(r(deviance)-`devpr2')
									local ud=`rev'
									local ud2=`rev2'
									* Ideally `dd'<0, but this may be unachievable near optimum,
									* so then can accept small deterioration in deviance
									if `dd'<`ltolera' {
										local improvd 1
										* provisionally accept
										local devnow=r(deviance)
										local devdiff `dd'
										scalar `b`v''=_b[`w`v'']
										scalar `m`v''=`mnew'
										scalar `s`v''=`snew'
										if `devdiff'>-`devcon' { local done2 1 }
										local dun 1 /* controls j3 loop */
									}
								}
								local j4=`j4'+1
							}
							* preserve best direction
							scalar `rev'=`ud'
							scalar `rev2'=`ud2'
							if "`trace'"!="" {
				    				noi di in gr "`v`v''" %4.0f `j2' /*
				     				*/ _col(16) "mnew=" in ye %9.0g `mnew' /*
				     				*/ _col(28) in gr " snew=" in ye %9.0g `snew' /*
				     				*/ _col(44) in gr " dd=" in ye %9.6f `dd' /*
				     				*/ in gr " improved=" in ye `improvd' in gr " stepsigns = " in ye `ud',`ud2'
				     			}
						}

					}
				}
				else {
					local jfail=`jfail'+1
					if `jfail'>10 {
						noi di in red "no convergence, too many fit failures"
						exit 498
					}
					if "`trace'"!="" {
						noi di in gr "fit failed, halving steps"
					}
					scalar `rev'=1
					scalar `rev2'=1
					scalar `delta'=`delta'/2
					scalar `delta2'=`delta2'/2
				}
				local j3=`j3'+1
			}
			local devpr2 `devnow'
			local totalit=`totalit'+1
			noi di in gr "Iteration `totalit':  Deviance = " /*
			 */ in ye %9.0g `devnow' in gr " (change = " /*
			 */ in ye %9.0g `devdiff' in gr ")"
			local j2=`j2'+1
			if !`dun' { local done2 1 } /* convergence failure */
		}
	    }
	    if `j2'==1 { local done 1 }
	    if `nx'>1 { local devdiff=(`devnow'-`devprev') }
	    if `devdiff'>10 {
		local done 1
		local converg 0
	    }
	    if `devdiff'>-`devcon' {
		local done 1
	    }
	    if "`trace'"!="" { noi di }
	    local devprev `devnow'
	    local j=`j'+1
	}
	if !`done' | (`devnow'>`devinit') { local converg 0 }
/*
	Tests for linearity for each nonlinear term
*/
	local terms: word count `wvars'
	local mdf=2*`terms'	/* model df for power terms + Taylor terms */
	local v 0
	while `v'<`nx' {
		local v=`v'+1
/*
	Replace wv with power terms for all but v'th predictor.
	Delete terms recursively from wvars.
*/
		local wv `wvars'
		frac_str `w`v'' `wv'
		local wv `s(new)'
		`cmd' `y' `v`v'' `wv' `covars' `wgt' if `touse', /*
		 */ `dead' `options'
		frac_dv `normal' "`wgt'" `nobs' `mnlnwt' `dist' /*
		 */ `glm' `xtgee' `qreg' "`scale'"
		local gain`v'=r(deviance)-`devnow'
		local lin`v'=_b[`v`v'']
		local se`v'=_se[`v`v'']
		if `gain`v''>0 {
			local d `gain`v''
			local n1 1
			local n2=`rdf'-`mdf'
			frac_pv `normal' "`wgt'" `nobs' `d' `n1' `n2'
			local P`v'=r(P)
		}
		else local P`v' .
	}
/*
	Final model.
	Approximation to SE(): crude. See Snedecor and Cochran 1967 p 470.
*/
	local zvars
	local z2vars
	local v 0
	while `v'<`nx' {
		local v=`v'+1
		tempname sem`v' ses`v'
		scalar `sem`v''=.
		scalar `ses`v''=.
		local zvars `zvars' `z`v''
		local z2vars `z2vars' `z2`v''
	}
	cap `cmd' `y' `wvars' `zvars' `z2vars' `covars' `wgt',`dead' `options'
	local xp
	local zvars
	local v 0
	while `v'<`nx' {
		local v=`v'+1
		cap scalar `sem`v''=_se[`z`v'']/abs(`b`v'')
		cap scalar `ses`v''=_se[`z2`v'']/abs(`b`v'')
		frac_mun `v`v''
		local vn `s(name)'
		gen double `vn'_1=`w`v''
		local xp`v' `vn'_1
		local xp `xp' `xp`v''
/*
	Make auxiliary variables permanent
*/
		rename `z`v'' `vn'_m
		rename `z2`v'' `vn'_s
		lab var `vn'_m "nprob auxil m(`v`v'')"
		lab var `vn'_s "nprob auxil s(`v`v'')"
		local zvars `zvars' `vn'_m `vn'_s
		local fvl `fvl' `xp`v'' `vn'_m `vn'_s
	}
}
local fvl `fvl' `covars'
qui `cmd' `y' `fvl' `wgt', `dead' `options'
di in gr _n "[Total iterations: `totalit']"
cap drop I__*	/* !!clean up -- nasty */
local i 0
while `i'<`nbase' {
	local i=`i'+1
	local j=`i'+`nx'
	est local fp_x`j': word `i' of `covars'
	est local fp_k`j' 1
	est local fp_a`j'
}
global S_1 `nobs'
global S_2 `devnow'

est local fp_xp `xp'
*est local fp_pwrs `pwrs'
est local fp_x `rhs'
est scalar fp_dev=`devnow'
est scalar fp_cnvg=`converg'

local v 0
while `v'<`nx' {
	local v=`v'+1
	est local fp_x`v' `v`v''	/* name of v`th predictor */
	est local fp_k`v'=`m`v''	/* m param for v'th predictor */
	est local fp_K`v'=`s`v''	/* s param for v'th predictor */
	est scalar fp_xb`v'=`xb`v''	/* mean of v'th predictor */
	est scalar fp_g`v'=`gain`v''	/* gain for v'th predictor */
	est scalar fp_P`v'=`P`v''	/* P-value for nonlinearity */
	est scalar fp_l`v'=`lin`v''	/* linear coefficient */
	est scalar fp_sl`v'=`se`v''	/* SE of linear coefficient */
	est scalar fp_a`v'=2		/* #auxiliaries for this var */
	global S_E_a`v' 2		/* double save ("bug" in -fracpred-) */

	local s=`sem`v''
	local S `s'
	local s=`ses`v''
	local S `S' `s'
	est local fp_s`v' `S'		/* se(params) for v'th predictor */
}
est local fp_base `covars'
est local fp_depv `y'
est scalar fp_dist=`dist'
est local fp_fvl `fvl'
est local fp_wgt "`weight'"
est local fp_exp "`exp'"
est local fp_glmw `glmwarn'
est scalar fp_nobs=`nobs'
est local fp_sfac `scalfac'
est scalar fp_nx=`nx'+`nbase'
est local fp_opts `dead' `options'
if "`nprob'"=="" {
	est local fp_t1t "Sigmoid (logistic) model"
}
else est local fp_t1t "Sigmoid (Normal integral) model"
est local fp_fn `nprob'
est local fp_cmd fracpoly
est local fp_cmd2 growth

if `rc'==0 { _rslt }
else error `rc'
end

program define _rslt
if "`e(fp_fn)'"=="" {
	local fname Logistic
	local fname2 "1/(1+exp(-s*(xvar-mean(xvar))-m)"
}
else {
	local fname Normal integral
	local fname2 " norm(s * (xvar-mean(xvar)) + m)"
}
di in gr _n "`fname' function regression model"
`e(cmd)'
local nx: word count `e(fp_x)'
local v 0
while `v'<`nx' {
	local v=`v'+1
	local var: word `v' of `e(fp_x)'
	local skip=13-length("`var'")
	frac_ddp e(fp_g`v') 3
	local nlindev `r(ddp)'

	di in gr "`var'" _skip(`skip') "|" /*
	 */ in ye _col(17) %9.0g e(fp_l`v') "  " %9.0g e(fp_sl`v') /*
	 */ " " %8.2f e(fp_l`v')/e(fp_sl`v') /*
	 */ in gr _col(49) "Nonlin. dev. " in ye "`nlindev'" /*
	 */ in gr _col(68) "(P = " in ye %5.3f e(fp_P`v') in gr ")"
	local p `e(fp_k`v')'
	local s: word 1 of `e(fp_s`v')'
	di in gr "           m |  " in ye %9.0g `p' "  " %9.0g /*
	 */ `s' " " %8.2f `p'/`s'
	local p `e(fp_K`v')'
	local s: word 2 of `e(fp_s`v')'
	di in gr "           s |  " in ye %9.0g `p' "  " %9.0g /*
	 */ `s' " " %8.2f `p'/`s'
	di in gr _dup(78) "-"
}
di in gr "Deviance:" in ye %9.3f e(fp_dev) in gr "." /*
 */ _col(22) "[Model for each xvar is `fname2']"
if e(fp_cnvg)==0 {
	di in red "divergence, or only partial convergence achieved"
	exit 430
}
end
