*! version 1.0.1 PR 15jan2007
program define xcor, rclass	// -correlate- with squared CC and Spearman and rankit options
syntax varlist(min=2 max=2) [if] [in]  [aw fw pw iw] , [ Spearman Rankit ]
if "`spearman'`rankit'"!="" & "`weight'"!="" & "`weight'"!="fweight" {
	di as err "weights not allowed"
	exit 198
}
if "`spearman'`rankit'"=="" local pearson "Pearson"
quietly {
	marksample touse
	local wgt [`weight'`exp']
	gettoken y x: varlist
	if "`weight'"=="fweight" {
		preserve
		drop if `touse'==0
		expand `exp'
		local wgt
	}
	if "`spearman'"!="" spearman `varlist' if `touse'
	else if "`rankit'"!="" {
		tempvar z1 z2
		local i 1
		tokenize `varlist'
		while "``i''"!="" {
			snscore `z`i'' = ``i'' if `touse'
			local ++i
		}
		correlate `z1' `z2' if `touse'
	}
	else correlate `varlist' `wgt' if `touse'
}
di as txt _n "`pearson'`spearman'`rankit' correlation = " %7.4f as res r(rho) ///
 as txt ", squared correlation = " %7.4f as res r(rho)^2 ///
 as txt " (" as res r(N) as txt " observations)"
return scalar N = r(N)
return scalar r = r(rho)
return scalar r2 = r(rho)^2
end

*! version 1.0.2 PR 14jan2007
program define snscore	/* scaled Normal scores or unscaled half-Normal scores */
syntax newvarname [= /exp] [if] [in], [ BY(varlist) MEDian noTies Full Half ]
if "`full'"!="" local factor 1
else local factor = sqrt(_pi/8)
tempvar touse GRV
quietly {
	marksample touse, novarlist
	if "`exp'"!="" {
		gen double `GRV' = (`exp') if `touse'==1
	}
	else {	/* take ordering of scores from existing observations */
		gen long `GRV'=_n
	}
	markout `touse' `GRV' `by'
	replace `touse'=. if `touse'==0
	sort `touse' `by' `GRV'
	if "`half'"=="" {
		by `touse' `by': gen double `varlist' = /*
		 */ `factor'*invnorm((_n-.375)/(_N+.25)) if `touse'==1
	}
	else {
/*
	McCullagh & Nelder 1989 p 407.
*/
		by `touse' `by': gen double `varlist' = /*
		 */ invnorm((_N+_n+.5)/(2*_N+1.125)) if `touse'==1
	}
	if "`ties'"!="noties" {
		if "`median'"=="" {
/*
	Replace Normal scores by their means in tie groups
*/
			by `touse' `by' `GRV': replace `varlist'=sum(`varlist') if `touse'==1
			by `touse' `by' `GRV': replace `varlist'=`varlist'[_N]/_N if `touse'==1
		}
		else {
/*
	Replace Normal scores by their medians in tie groups
	!! probably could be done more elegantly without use of -byvar-
*/
			byvar `touse' `by' `GRV', r(p50) generate: /*
			 */ summarize `varlist' if `touse'==1, detail
			local sumvar `r(R_1)'
			replace `varlist'=`sumvar'
			drop `sumvar'
		}
	}
}
end
exit
* Alternative formula:
		by `touse' `by': replace `varlist' = /*
		 */ invnorm(.5+.5*(`rank'-.375)/(_N+.25)) if `touse'==1
